@@ -1,13 +1,34 @@
 import textwrap
 from typing import Iterable, Sequence
 
+from ai.backend.client.output.fields import group_fields
+from ai.backend.client.output.types import FieldSpec
 from .base import api_function, BaseFunction
-from ..request import Request
+from ..session import api_session
 
 __all__ = (
     'Group',
 )
 
+_default_list_fields = (
+    group_fields['id'],
+    group_fields['name'],
+    group_fields['is_active'],
+    group_fields['created_at'],
+    group_fields['integration_id'],
+)
+_default_detail_fields = (
+    group_fields['id'],
+    group_fields['name'],
+    group_fields['description'],
+    group_fields['is_active'],
+    group_fields['created_at'],
+    group_fields['domain_name'],
+    group_fields['total_resource_slots'],
+    group_fields['allowed_vfolder_hosts'],
+    group_fields['integration_id'],
+)
+
 
 class Group(BaseFunction):
     """
@@ -22,38 +43,69 @@ class Group(BaseFunction):
 
     @api_function
     @classmethod
-    async def list(cls, domain_name: str,
-                   fields: Iterable[str] = None) -> Sequence[dict]:
+    async def from_name(
+        cls,
+        name: str,
+        *,
+        fields: Iterable[str] = None,
+        domain_name: str = None,
+    ) -> Sequence[dict]:
+        """
+        Find the group(s) by its name.
+        It may return multiple groups when there are groups with the same name
+        in different domains and it is invoked with a super-admin account
+        without setting the domain name.
+
+        :param domain_name: Name of domain to get groups from.
+        :param fields: Per-group query fields to fetch.
+        """
+        if fields is None:
+            fields = _default_detail_fields
+        query = textwrap.dedent("""\
+            query($name: String!, $domain_name: String) {
+                groups_by_name(name: $name, domain_name: $domain_name) {$fields}
+            }
+        """)
+        query = query.replace('$fields', ' '.join(fields))
+        variables = {
+            'name': name,
+            'domain_name': domain_name,
+        }
+        data = await api_session.get().Admin._query(query, variables)
+        return data['groups_by_name']
+
+    @api_function
+    @classmethod
+    async def list(
+        cls,
+        domain_name: str,
+        fields: Sequence[FieldSpec] = _default_list_fields,
+    ) -> Sequence[dict]:
         """
         Fetches the list of groups.
 
         :param domain_name: Name of domain to list groups.
-        :param fields: Additional per-group query fields to fetch.
+        :param fields: Per-group query fields to fetch.
         """
         if fields is None:
-            fields = ('id', 'name', 'description', 'is_active',
-                      'created_at', 'domain_name',
-                      'total_resource_slots', 'allowed_vfolder_hosts',
-                      'integration_id')
+            fields = _default_list_fields
         query = textwrap.dedent("""\
             query($domain_name: String) {
                 groups(domain_name: $domain_name) {$fields}
             }
         """)
-        query = query.replace('$fields', ' '.join(fields))
+        query = query.replace('$fields', ' '.join(f.field_ref for f in fields))
         variables = {'domain_name': domain_name}
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
+        data = await api_session.get().Admin._query(query, variables)
         return data['groups']
 
     @api_function
     @classmethod
-    async def detail(cls, gid: str, fields: Iterable[str] = None) -> Sequence[dict]:
+    async def detail(
+        cls,
+        gid: str,
+        fields: Sequence[FieldSpec] = _default_detail_fields,
+    ) -> dict:
         """
         Fetch information of a group with group ID.
 
@@ -61,22 +113,15 @@ class Group(BaseFunction):
         :param fields: Additional per-group query fields to fetch.
         """
         if fields is None:
-            fields = ('id', 'name', 'description', 'is_active', 'created_at', 'domain_name',
-                      'total_resource_slots', 'allowed_vfolder_hosts', 'integration_id')
+            fields = _default_detail_fields
         query = textwrap.dedent("""\
             query($gid: UUID!) {
                 group(id: $gid) {$fields}
             }
         """)
-        query = query.replace('$fields', ' '.join(fields))
+        query = query.replace('$fields', ' '.join(f.field_ref for f in fields))
         variables = {'gid': gid}
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
+        data = await api_session.get().Admin._query(query, variables)
         return data['group']
 
     @api_function
@@ -91,7 +136,7 @@ class Group(BaseFunction):
         You need an admin privilege for this operation.
         """
         if fields is None:
-            fields = ('id', 'domain_name', 'name',)
+            fields = ('id', 'domain_name', 'name')
         query = textwrap.dedent("""\
             mutation($name: String!, $input: GroupInput!) {
                 create_group(name: $name, props: $input) {
@@ -111,13 +156,7 @@ class Group(BaseFunction):
                 'integration_id': integration_id,
             },
         }
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
+        data = await api_session.get().Admin._query(query, variables)
         return data['create_group']
 
     @api_function
@@ -149,13 +188,7 @@ class Group(BaseFunction):
                 'integration_id': integration_id,
             },
         }
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
+        data = await api_session.get().Admin._query(query, variables)
         return data['modify_group']
 
     @api_function
@@ -172,13 +205,7 @@ class Group(BaseFunction):
             }
         """)
         variables = {'gid': gid}
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
+        data = await api_session.get().Admin._query(query, variables)
         return data['delete_group']
 
     @api_function
@@ -195,13 +222,7 @@ class Group(BaseFunction):
             }
         """)
         variables = {'gid': gid}
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
+        data = await api_session.get().Admin._query(query, variables)
         return data['purge_group']
 
     @api_function
@@ -226,13 +247,7 @@ class Group(BaseFunction):
                 'user_uuids': user_uuids,
             },
         }
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
+        data = await api_session.get().Admin._query(query, variables)
         return data['modify_group']
 
     @api_function
@@ -257,11 +272,5 @@ class Group(BaseFunction):
                 'user_uuids': user_uuids,
             },
         }
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
+        data = await api_session.get().Admin._query(query, variables)
         return data['modify_group']
