@@ -2,13 +2,36 @@ import json
 import textwrap
 from typing import Iterable, Mapping, Sequence
 
+from ai.backend.client.output.fields import scaling_group_fields
+from ai.backend.client.output.types import FieldSpec
 from .base import api_function, BaseFunction
 from ..request import Request
+from ..session import api_session
 
 __all__ = (
     'ScalingGroup',
 )
 
+_default_list_fields = (
+    scaling_group_fields['name'],
+    scaling_group_fields['description'],
+    scaling_group_fields['is_active'],
+    scaling_group_fields['created_at'],
+    scaling_group_fields['driver'],
+    scaling_group_fields['scheduler'],
+)
+
+_default_detail_fields = (
+    scaling_group_fields['name'],
+    scaling_group_fields['description'],
+    scaling_group_fields['is_active'],
+    scaling_group_fields['created_at'],
+    scaling_group_fields['driver'],
+    scaling_group_fields['driver_opts'],
+    scaling_group_fields['scheduler'],
+    scaling_group_fields['scheduler_opts'],
+)
+
 
 class ScalingGroup(BaseFunction):
     """
@@ -29,23 +52,25 @@ class ScalingGroup(BaseFunction):
         List available scaling groups for the current user,
         considering the user, the user's domain, and the designated user group.
         """
-        rqst = Request('GET', '/scaling-groups',
-                       params={'group': group})
+        rqst = Request(
+            'GET', '/scaling-groups',
+            params={'group': group},
+        )
         async with rqst.fetch() as resp:
-            return await resp.json()
+            data = await resp.json()
+            print(data)
+            return data['scaling_groups']
 
     @api_function
     @classmethod
-    async def list(cls, fields: Iterable[str] = None) -> Sequence[dict]:
+    async def list(
+        cls,
+        fields: Sequence[FieldSpec] = _default_list_fields,
+    ) -> Sequence[dict]:
         """
         List available scaling groups for the current user,
         considering the user, the user's domain, and the designated user group.
         """
-        if fields is None:
-            fields = ('name', 'description', 'is_active',
-                      'created_at',
-                      'driver', 'driver_opts',
-                      'scheduler', 'scheduler_opts',)
         query = textwrap.dedent("""\
             query($is_active: Boolean) {
                 scaling_groups(is_active: $is_active) {
@@ -53,46 +78,33 @@ class ScalingGroup(BaseFunction):
                 }
             }
         """)
-        query = query.replace('$fields', ' '.join(fields))
+        query = query.replace('$fields', ' '.join(f.field_ref for f in fields))
         variables = {'is_active': None}
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
-            return data['scaling_groups']
+        data = await api_session.get().Admin._query(query, variables)
+        return data['scaling_groups']
 
     @api_function
     @classmethod
-    async def detail(cls, name: str, fields: Iterable[str] = None) -> Sequence[dict]:
+    async def detail(
+        cls,
+        name: str,
+        fields: Sequence[FieldSpec] = _default_detail_fields,
+    ) -> dict:
         """
         Fetch information of a scaling group by name.
 
         :param name: Name of the scaling group.
         :param fields: Additional per-scaling-group query fields.
         """
-        if fields is None:
-            fields = ('name', 'description', 'is_active',
-                      'created_at',
-                      'driver', 'driver_opts',
-                      'scheduler', 'scheduler_opts',)
         query = textwrap.dedent("""\
             query($name: String) {
                 scaling_group(name: $name) {$fields}
             }
         """)
-        query = query.replace('$fields', ' '.join(fields))
+        query = query.replace('$fields', ' '.join(f.field_ref for f in fields))
         variables = {'name': name}
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
-            return data['scaling_group']
+        data = await api_session.get().Admin._query(query, variables)
+        return data['scaling_group']
 
     @api_function
     @classmethod
@@ -106,7 +118,7 @@ class ScalingGroup(BaseFunction):
         if fields is None:
             fields = ('name',)
         query = textwrap.dedent("""\
-            mutation($name: String!, $input: ScalingGroupInput!) {
+            mutation($name: String!, $input: CreateScalingGroupInput!) {
                 create_scaling_group(name: $name, props: $input) {
                     ok msg scaling_group {$fields}
                 }
@@ -124,14 +136,8 @@ class ScalingGroup(BaseFunction):
                 'scheduler_opts': json.dumps(scheduler_opts),
             },
         }
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
-            return data['create_scaling_group']
+        data = await api_session.get().Admin._query(query, variables)
+        return data['create_scaling_group']
 
     @api_function
     @classmethod
@@ -158,19 +164,13 @@ class ScalingGroup(BaseFunction):
                 'description': description,
                 'is_active': is_active,
                 'driver': driver,
-                'driver_opts': json.dumps(driver_opts),
+                'driver_opts': None if driver_opts is None else json.dumps(driver_opts),
                 'scheduler': scheduler,
-                'scheduler_opts': json.dumps(scheduler_opts),
+                'scheduler_opts': None if scheduler_opts is None else json.dumps(scheduler_opts),
             },
         }
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
-            return data['modify_scaling_group']
+        data = await api_session.get().Admin._query(query, variables)
+        return data['modify_scaling_group']
 
     @api_function
     @classmethod
@@ -186,14 +186,8 @@ class ScalingGroup(BaseFunction):
             }
         """)
         variables = {'name': name}
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
-            return data['delete_scaling_group']
+        data = await api_session.get().Admin._query(query, variables)
+        return data['delete_scaling_group']
 
     @api_function
     @classmethod
@@ -213,14 +207,8 @@ class ScalingGroup(BaseFunction):
             }
         """)
         variables = {'scaling_group': scaling_group, 'domain': domain}
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
-            return data['associate_scaling_group_with_domain']
+        data = await api_session.get().Admin._query(query, variables)
+        return data['associate_scaling_group_with_domain']
 
     @api_function
     @classmethod
@@ -240,14 +228,8 @@ class ScalingGroup(BaseFunction):
             }
         """)
         variables = {'scaling_group': scaling_group, 'domain': domain}
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
-            return data['disassociate_scaling_group_with_domain']
+        data = await api_session.get().Admin._query(query, variables)
+        return data['disassociate_scaling_group_with_domain']
 
     @api_function
     @classmethod
@@ -265,14 +247,8 @@ class ScalingGroup(BaseFunction):
             }
         """)
         variables = {'domain': domain}
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
-            return data['disassociate_all_scaling_groups_with_domain']
+        data = await api_session.get().Admin._query(query, variables)
+        return data['disassociate_all_scaling_groups_with_domain']
 
     @api_function
     @classmethod
@@ -284,7 +260,7 @@ class ScalingGroup(BaseFunction):
         :param group_id: The ID of a group.
         """
         query = textwrap.dedent("""\
-            mutation($scaling_group: String!, $user_group: String!) {
+            mutation($scaling_group: String!, $user_group: UUID!) {
                 associate_scaling_group_with_user_group(
                         scaling_group: $scaling_group, user_group: $user_group) {
                     ok msg
@@ -292,14 +268,8 @@ class ScalingGroup(BaseFunction):
             }
         """)
         variables = {'scaling_group': scaling_group, 'user_group': group_id}
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
-            return data['associate_scaling_group_with_user_group']
+        data = await api_session.get().Admin._query(query, variables)
+        return data['associate_scaling_group_with_user_group']
 
     @api_function
     @classmethod
@@ -319,14 +289,8 @@ class ScalingGroup(BaseFunction):
             }
         """)
         variables = {'scaling_group': scaling_group, 'user_group': group_id}
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
-            return data['disassociate_scaling_group_with_user_group']
+        data = await api_session.get().Admin._query(query, variables)
+        return data['disassociate_scaling_group_with_user_group']
 
     @api_function
     @classmethod
@@ -337,18 +301,12 @@ class ScalingGroup(BaseFunction):
         :param group_id: The ID of a group.
         """
         query = textwrap.dedent("""\
-            mutation($group_id: String!) {
+            mutation($group_id: UUID!) {
                 disassociate_all_scaling_groups_with_group(user_group: $group_id) {
                     ok msg
                 }
             }
         """)
         variables = {'group_id': group_id}
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
-            return data['disassociate_all_scaling_groups_with_group']
+        data = await api_session.get().Admin._query(query, variables)
+        return data['disassociate_all_scaling_groups_with_group']
