@@ -12,19 +12,21 @@ from typing import (
     Dict,
     Iterable,
     List,
+    Literal,
     Mapping,
     Optional,
     Sequence,
     Union,
     cast,
 )
-from typing_extensions import Literal  # for Python 3.7
 from uuid import UUID
 
 import aiohttp
 from aiohttp import hdrs
 from tqdm import tqdm
 
+from ai.backend.client.output.fields import session_fields
+from ai.backend.client.output.types import FieldSpec, PaginatedResult
 from .base import api_function, BaseFunction
 from ..compat import current_loop
 from ..config import DEFAULT_CHUNK_SIZE
@@ -45,6 +47,16 @@ __all__ = (
     'ComputeSession',
 )
 
+_default_list_fields = (
+    session_fields['session_id'],
+    session_fields['image'],
+    session_fields['type'],
+    session_fields['status'],
+    session_fields['status_info'],
+    session_fields['status_changed'],
+    session_fields['result'],
+)
+
 
 def drop(d: Mapping[str, Any], value_to_drop: Any) -> Mapping[str, Any]:
     modified: Dict[str, Any] = {}
@@ -87,35 +99,30 @@ class ComputeSession(BaseFunction):
         status: str = None,
         access_key: str = None,
         *,
-        fields: Sequence[str] = None,
+        fields: Sequence[FieldSpec] = _default_list_fields,
+        page_offset: int = 0,
         page_size: int = 20,
-    ) -> AsyncIterator[dict]:
+        filter: str = None,
+        order: str = None,
+    ) -> PaginatedResult[dict]:
         """
         Fetches the list of users. Domain admins can only get domain users.
 
         :param is_active: Fetches active or inactive users only if not None.
         :param fields: Additional per-user query fields to fetch.
         """
-        if fields is None:
-            fields = [
-                'task_id',
-                'image',
-                'type',
-                'status',
-                'status_info',
-                'last_updated',
-                'result',
-            ]
-        async for item in generate_paginated_results(
+        return await generate_paginated_results(
             'compute_session_list',
             {
                 'status': (status, 'String'),
                 'access_key': (access_key, 'String'),
+                'filter': (filter, 'String'),
+                'order': (order, 'String'),
             },
             fields,
+            page_offset=page_offset,
             page_size=page_size,
-        ):
-            yield item
+        )
 
     @api_function
     @classmethod
@@ -128,7 +135,7 @@ class ComputeSession(BaseFunction):
     @classmethod
     async def get_task_logs(
         cls, task_id: str, *,
-        chunk_size: int = 8192
+        chunk_size: int = 8192,
     ) -> AsyncIterator[bytes]:
         prefix = get_naming(api_session.get().api_version, 'path')
         rqst = Request('GET', f'/{prefix}/_/logs', params={
@@ -167,6 +174,7 @@ class ComputeSession(BaseFunction):
         scaling_group: str = None,
         owner_access_key: str = None,
         preopen_ports: List[int] = None,
+        assign_agent: List[str] = None,
     ) -> ComputeSession:
         """
         Get-or-creates a compute session.
@@ -271,6 +279,7 @@ class ComputeSession(BaseFunction):
             params['config'].update({
                 'mount_map': mount_map,
                 'preopen_ports': preopen_ports,
+                'agentList': assign_agent,
             })
             params.update({
                 'starts_at': starts_at,
@@ -854,7 +863,7 @@ class ComputeSession(BaseFunction):
                 params={
                     **self.get_session_identity_params(),
                     'scope': scope,
-                }
+                },
             )
         else:
             assert self.name is not None
