@@ -1,28 +1,73 @@
+from __future__ import annotations
+
+import enum
 import textwrap
-from typing import AsyncIterator, Iterable, Sequence, Union
+from typing import (
+    Iterable,
+    Sequence,
+    Union,
+)
 import uuid
 
+from ai.backend.client.auth import AuthToken, AuthTokenTypes
+from ai.backend.client.request import Request
+from ai.backend.client.session import api_session
+from ai.backend.client.output.fields import user_fields
+from ai.backend.client.output.types import FieldSpec, PaginatedResult
+from ai.backend.client.pagination import generate_paginated_results
 from .base import api_function, BaseFunction
-from ..auth import AuthToken, AuthTokenTypes
-from ..request import Request
-from ..session import api_session
-from ..pagination import generate_paginated_results
 
 __all__ = (
     'User',
+    'UserStatus',
+    'UserRole',
 )
 
+
 _default_list_fields = (
-    'uuid',
-    'role',
-    'username',
-    'email',
-    'is_active',
-    'created_at',
-    'domain_name',
-    'groups',
+    user_fields['uuid'],
+    user_fields['role'],
+    user_fields['username'],
+    user_fields['email'],
+    user_fields['is_active'],
+    user_fields['created_at'],
+    user_fields['domain_name'],
+    user_fields['groups'],
 )
 
+_default_detail_fields = (
+    user_fields['uuid'],
+    user_fields['username'],
+    user_fields['email'],
+    user_fields['need_password_change'],
+    user_fields['status'],
+    user_fields['status_info'],
+    user_fields['created_at'],
+    user_fields['domain_name'],
+    user_fields['role'],
+    user_fields['groups'],
+)
+
+
+class UserRole(str, enum.Enum):
+    """
+    The role (privilege level) of users.
+    """
+    SUPERADMIN = 'superadmin'
+    ADMIN = 'admin'
+    USER = 'user'
+    MONITOR = 'monitor'
+
+
+class UserStatus(enum.Enum):
+    """
+    The detailed status of users to represent the signup process and account lifecycles.
+    """
+    ACTIVE = 'active'
+    INACTIVE = 'inactive'
+    DELETED = 'deleted'
+    BEFORE_VERIFICATION = 'before-verification'
+
 
 class User(BaseFunction):
     """
@@ -61,7 +106,7 @@ class User(BaseFunction):
         cls,
         status: str = None,
         group: str = None,
-        fields: Sequence[str] = _default_list_fields,
+        fields: Sequence[FieldSpec] = _default_list_fields,
     ) -> Sequence[dict]:
         """
         Fetches the list of users. Domain admins can only get domain users.
@@ -76,19 +121,13 @@ class User(BaseFunction):
                 users(status: $status, group_id: $group) {$fields}
             }
         """)
-        query = query.replace('$fields', ' '.join(fields))
+        query = query.replace('$fields', ' '.join(f.field_ref for f in fields))
         variables = {
             'status': status,
             'group': group,
         }
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
-            return data['users']
+        data = await api_session.get().Admin._query(query, variables)
+        return data['users']
 
     @api_function
     @classmethod
@@ -97,9 +136,12 @@ class User(BaseFunction):
         status: str = None,
         group: str = None,
         *,
-        fields: Sequence[str] = _default_list_fields,
+        fields: Sequence[FieldSpec] = _default_list_fields,
+        page_offset: int = 0,
         page_size: int = 20,
-    ) -> AsyncIterator[dict]:
+        filter: str = None,
+        order: str = None,
+    ) -> PaginatedResult[dict]:
         """
         Fetches the list of users. Domain admins can only get domain users.
 
@@ -108,20 +150,26 @@ class User(BaseFunction):
         :param group: Fetch users in a specific group.
         :param fields: Additional per-user query fields to fetch.
         """
-        async for item in generate_paginated_results(
+        return await generate_paginated_results(
             'user_list',
             {
                 'status': (status, 'String'),
                 'group_id': (group, 'UUID'),
+                'filter': (filter, 'String'),
+                'order': (order, 'String'),
             },
             fields,
+            page_offset=page_offset,
             page_size=page_size,
-        ):
-            yield item
+        )
 
     @api_function
     @classmethod
-    async def detail(cls, email: str = None, fields: Iterable[str] = None) -> Sequence[dict]:
+    async def detail(
+        cls,
+        email: str = None,
+        fields: Sequence[FieldSpec] = _default_detail_fields,
+    ) -> Sequence[dict]:
         """
         Fetch information of a user. If email is not specified,
         requester's information will be returned.
@@ -129,9 +177,6 @@ class User(BaseFunction):
         :param email: Email of the user to fetch.
         :param fields: Additional per-user query fields to fetch.
         """
-        if fields is None:
-            fields = ('uuid', 'username', 'email', 'need_password_change', 'status', 'status_info'
-                      'created_at', 'domain_name', 'role')
         if email is None:
             query = textwrap.dedent("""\
                 query {
@@ -144,28 +189,17 @@ class User(BaseFunction):
                     user(email: $email) {$fields}
                 }
             """)
-        query = query.replace('$fields', ' '.join(fields))
+        query = query.replace('$fields', ' '.join(f.field_ref for f in fields))
         variables = {'email': email}
-        rqst = Request('POST', '/admin/graphql')
-        if email is None:
-            rqst.set_json({
-                'query': query,
-            })
-        else:
-            rqst.set_json({
-                'query': query,
-                'variables': variables,
-            })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
-            return data['user']
+        data = await api_session.get().Admin._query(query, variables if email is not None else None)
+        return data['user']
 
     @api_function
     @classmethod
     async def detail_by_uuid(
         cls,
         user_uuid: Union[str, uuid.UUID] = None,
-        fields: Iterable[str] = None,
+        fields: Sequence[FieldSpec] = _default_detail_fields,
     ) -> Sequence[dict]:
         """
         Fetch information of a user by user's uuid. If user_uuid is not specified,
@@ -174,9 +208,6 @@ class User(BaseFunction):
         :param user_uuid: UUID of the user to fetch.
         :param fields: Additional per-user query fields to fetch.
         """
-        if fields is None:
-            fields = ('uuid', 'username', 'email', 'need_password_change', 'status', 'status_info',
-                      'created_at', 'domain_name', 'role')
         if user_uuid is None:
             query = textwrap.dedent("""\
                 query {
@@ -189,40 +220,33 @@ class User(BaseFunction):
                     user_from_uuid(user_id: $user_id) {$fields}
                 }
             """)
-        query = query.replace('$fields', ' '.join(fields))
+        query = query.replace('$fields', ' '.join(f.field_ref for f in fields))
         variables = {'user_id': str(user_uuid)}
-        rqst = Request('POST', '/admin/graphql')
-        if user_uuid is None:
-            rqst.set_json({
-                'query': query,
-            })
-        else:
-            rqst.set_json({
-                'query': query,
-                'variables': variables,
-            })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
-            return data['user_from_uuid']
+        data = await api_session.get().Admin._query(query, variables if user_uuid is not None else None)
+        return data['user_from_uuid']
 
     @api_function
     @classmethod
     async def create(
         cls,
         domain_name: str,
-        email: str, password: str, username: str = None, full_name: str = None,
-        role: str = 'user', status: bool = True,
+        email: str,
+        password: str,
+        username: str = None,
+        full_name: str = None,
+        role: UserRole | str = UserRole.USER,
+        status: UserStatus | str = UserStatus.ACTIVE,
         need_password_change: bool = False,
         description: str = '',
         group_ids: Iterable[str] = None,
-        fields: Iterable[str] = None
+        fields: Iterable[str] = None,
     ) -> dict:
         """
         Creates a new user with the given options.
         You need an admin privilege for this operation.
         """
         if fields is None:
-            fields = ('domain_name', 'email', 'username',)
+            fields = ('domain_name', 'email', 'username')
         query = textwrap.dedent("""\
             mutation($email: String!, $input: UserInput!) {
                 create_user(email: $email, props: $input) {
@@ -237,22 +261,16 @@ class User(BaseFunction):
                 'password': password,
                 'username': username,
                 'full_name': full_name,
-                'role': role,
-                'status': status,
+                'role': role.value if isinstance(role, UserRole) else role,
+                'status': status.value if isinstance(status, UserStatus) else status,
                 'need_password_change': need_password_change,
                 'description': description,
                 'domain_name': domain_name,
                 'group_ids': group_ids,
             },
         }
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
-            return data['create_user']
+        data = await api_session.get().Admin._query(query, variables)
+        return data['create_user']
 
     @api_function
     @classmethod
@@ -262,11 +280,12 @@ class User(BaseFunction):
         password: str = None, username: str = None,
         full_name: str = None,
         domain_name: str = None,
-        role: str = None, status: bool = None,
+        role: UserRole | str = UserRole.USER,
+        status: UserStatus | str = UserStatus.ACTIVE,
         need_password_change: bool = None,
         description: str = None,
         group_ids: Iterable[str] = None,
-        fields: Iterable[str] = None
+        fields: Iterable[str] = None,
     ) -> dict:
         """
         Update existing user.
@@ -286,21 +305,15 @@ class User(BaseFunction):
                 'username': username,
                 'full_name': full_name,
                 'domain_name': domain_name,
-                'role': role,
-                'status': status,
+                'role': role.value if isinstance(role, UserRole) else role,
+                'status': status.value if isinstance(status, UserStatus) else status,
                 'need_password_change': need_password_change,
                 'description': description,
                 'group_ids': group_ids,
             },
         }
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
-            return data['modify_user']
+        data = await api_session.get().Admin._query(query, variables)
+        return data['modify_user']
 
     @api_function
     @classmethod
@@ -316,14 +329,8 @@ class User(BaseFunction):
             }
         """)
         variables = {'email': email}
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
-            return data['delete_user']
+        data = await api_session.get().Admin._query(query, variables)
+        return data['delete_user']
 
     @api_function
     @classmethod
@@ -348,11 +355,5 @@ class User(BaseFunction):
                 'purge_shared_vfolders': purge_shared_vfolders,
             },
         }
-        rqst = Request('POST', '/admin/graphql')
-        rqst.set_json({
-            'query': query,
-            'variables': variables,
-        })
-        async with rqst.fetch() as resp:
-            data = await resp.json()
-            return data['purge_user']
+        data = await api_session.get().Admin._query(query, variables)
+        return data['purge_user']
